10000 REMark '' Structured SuperBasic Filter
10010 REMark '' Author:       Timothy Swenson
10020 REMark '' Date Created: 16 Oct 1990
10030 REMark '' Date Revised: 4 Oct 2021
10040 REMark '' Version:      2.7.2b
10050 REMark ''
10060 REMark ''   This program takes in SSB code and outputs a file
10070 REMark ''     that is runnable in SuperBasic
10080 REMark ''
10090 REMark '' Requires:  Environment Variables (env_bin)
10100 REMark ''            TURBO Toolkit (TTK) 3f32
10110 REMark ''            ToolKit II (TKII)
10120 REMark ''
10130 REMark '' Default values:
10140 REMark ''    Number of Labels    30
10150 REMark ''    Length of Labels    16
10160 REMark ''    Input File Ext.     _ssb
10170 REMark ''    Output File Ext.    _bas
10180 REMark ''    Working Dir.        (blank)
10190 REMark ''    2nd Working Dir.    (blank)
10200 REMark ''    Line Num. Start     100
10210 REMark ''    Line Num. Delta     10
10220 REMark ''    BAS Overwrite       (blank)
10230 REMark ''    Language            English ("E")
10240 REMark ''    Ignore Blank lines  Y
10250 REMark ''
10260 REMark ''   Channels :
10270 REMark ''               3 - Screen
10280 REMark ''               4 - Input File
10290 REMark ''               5 - Output File
10300 REMark ''
10310 :
10320 :
10330 REMark '' TURBO Defines
10340 :
10350 TURBO_taskn     "ssb 2.7.2b"
10360 TURBO_objfil    "ram1_ssb272b_exe"
10370 TURBO_windo     0
10380 TURBO_repfil    "ram1_ssb272b_err"
10390 TURBO_objdat    400
10400 TURBO_buffersz  400
10410 TURBO_optim     0
10420 TURBO_diags     2
10430 TURBO_locstr    1
10440 TURBO_struct    0
10450 :
10460 IMPLICIT% d_labels, d_length, d_start, d_delta, label_var, define_var
10470 IMPLICIT% num_count, file_num, COUNT, x, y, temp, line_num_d, line_delta
10480 IMPLICIT% line_num
10490 :
10500 :
10510 :
10520 REMark '' Default Values
10530 :
10540 d_labels        = 30
10550 d_length        = 16
10560 d_in$           = "_ssb"
10570 d_out$          = "_bas"
10580 d_start         = 100
10590 d_delta         = 10
10600 d_working$      = ""
10610 d_work2$        = ""
10620 d_over$         = ""
10630 d_lang$         = "E"
10640 d_ignore$       = "Y"
10650 :
10660 :
10670 :
10680 REMark '' Init Variables
10690 DIM out_file$(100), command$(100), file$(100), in_file$(100), temp$(100)
10700 DIM line_num$(10), line_delta$(10), argfile$(100), in$(1000), in2$(1000)
10710 DIM label$(d_labels,d_length)
10720 DIM label(d_labels)
10730 DIM defn$(d_labels,d_length)
10740 label_var = 1
10750 define_var = 1
10760 num_count = 0
10770 file_num = 5
10780 out_file$ = ""
10790 :
10800 :
10810 :
10820 REMark ''  This section adds support for the environment
10830 REMark ''    variables (ENV_BIN).
10840 REMark ''
10850 :
10860 t_in$ = GETENV$("SSBINPUT")
10870 IF t_in$ <> "" THEN d_in$ = t_in$
10880 t_out$ = GETENV$("SSBOUTPUT")
10890 IF t_out$ <> "" THEN d_out$ = t_out$
10900 t_start$ = GETENV$("SSBSTART")
10910 IF t_start$ <> "" THEN d_start = t_start$
10920 t_delta$ = GETENV$("SSBINC")
10930 IF t_delta$ <> "" THEN d_delta = t_delta$
10940 t_working$ = GETENV$("SSBWORK")
10950 IF t_working$ <> "" THEN d_working$ = t_working$
10960 t_work2$ = GETENV$("SSBWORK2")
10970 IF t_work2$ <> "" THEN d_work2$ = t_work2$
10980 t_over$ = GETENV$("SSBOVER")
10990 IF t_over$ <> "" THEN d_over$ = t_over$
11000 t_lang$ = GETENV$("SSBLANG")
11010 IF t_lang$ <> "" THEN d_lang$ = t_lang$
11020 t_ignore$ = GETENV$("SSBIGNORE_BLANKS")
11030 IF t_ignore$ <> "" THEN d_ignore$ = t_ignore$
11040 :
11050 REMark '' Probably need to add some error checking here
11060 :
11070 :
11080 REMark '' Add in code for language support
11090 MISTake #INCLUDE ram1_ssblang_ssb
11100 :
11110 :
11120 :
11130 OPEN #3,"con_300x160a75x10_32"
11140 BORDER #3,2,4
11150 PAPER #3,0 : INK #3,7 :  CLS #3
11160 :
11170 :
11180 REMark '' Print opening Window
11190 REMark ''  STRUCTURED SUPERBASIC FILTER
11200 PRINT #3,"     ";lngarray$(1)
11210 INK #3,2
11220 REMark ''  Version 2.7.2
11230 PRINT #3,"            ";lngarray$(2)
11240 REMark ''  By Timothy Swenson
11250 PRINT #3,"         ";lngarray$(3)
11260 PRINT #3
11270 INK #3,4
11280 :
11290 REMark '' If Working Dir 2 is blank, set to Working Dir
11300 IF d_work2$ = "" THEN d_work2$ = d_working$
11310 :
11320 REMark '' Get Command Line Args from Turbo
11330 command$ = OPTION_CMD$
11340 :
11350 REMark '' Check for arguement of only spaces
11360 COUNT = 0
11370 y = LEN(command$)
11380 IF y > 0 THEN
11390   FOR x = 1 TO y
11400     IF command$(x) = " " THEN COUNT=COUNT+1
11410   NEXT x
11420   IF COUNT = y THEN command$=""
11430 END IF
11440 :
11450 IF command$ = "" THEN
11460 :
11470   REPeat input_loop
11480     REMark '' Enter Input File
11490     PRINT #3,lngarray$(4);d_in$;") "
11500     INK #3,7
11510     INPUT #3,file$
11520 :
11530     REMark '' If enter a null string, exit program
11540     IF file$ = "" THEN abort_out
11550 :
11560     REMark '' Set Ink Color to Red for Errors
11570     INK #3,2
11580 :
11590     in_file$ = d_working$&file$&d_in$
11600     temp = FTEST(in_file$)
11610     REMark '' File Exits
11620     IF temp = 0 THEN EXIT input_loop
11630 :
11640     IF temp = -9 THEN
11650       BEEP 1000,10
11660         REMark '' ERROR - File In Use
11670         PRINT #3,"      ";lngarray$(5)
11680     END IF
11690 :
11700     IF temp = -7 THEN
11710       BEEP 1000,10
11720       REMark '' ERROR - File Does Not Exist
11730       PRINT #3,"      ";lngarray$(6)
11740     END IF
11750 :
11760     IF temp = -11 THEN
11770       BEEP 1000,10
11780       REMark '' ERROR - Bad File Name
11790       PRINT #3,"      ";lngarray$(7)
11800     END IF
11810 :
11820   END REPeat input_loop
11830 :
11840   REMark '' Check for extension.  If no ext. add _ssb
11850   REPeat output_loop
11860     REMark '' Enter Output File: Default
11870     INK #3,4
11880     PRINT #3,lngarray$(8);d_work2$&file$&d_out$;") "
11890     INK #3,7
11900     INPUT #3,out_file$
11910 :
11920     IF out_file$ = "" THEN
11930       out_file$ = d_work2$ & file$ & d_out$
11940     ELSE
11950       out_file$ = d_work2$ & out_file$ & d_out$
11960     END IF
11970 :
11980     REMark '' Set Ink Color to Red for Errors
11990     INK #3,2
12000 :
12010     temp = FTEST(out_file$)
12020     REMark '' If file does not exist
12030     IF temp = -7 THEN EXIT output_loop
12040     IF temp = -9 THEN
12050       BEEP 1000,10
12060       REMark '' ERROR - File In Use
12070       PRINT #3,"       ";lngarray$(5)
12080     END IF
12090 :
12100     IF temp = -11 THEN
12110       BEEP 1000,10
12120       REMark '' ERROR - Bad File Name
12130       PRINT #3,"       ";lngarray$(7)
12140     END IF
12150 :
12160     IF temp = 0 THEN
12170       BEEP 1000,10
12180       REMark '' ERROR - File Exists
12190       PRINT #3,"       ";lngarray$(9)
12200       REMark '' OK to Delete (Y/N)
12210       PRINT #3,"        ";lngarray$(10)
12220       temp$ = INKEY$(#3,-1)
12230       REMark ''
12240       IF temp$ = "y" OR temp$="Y" OR temp$="S" OR temp$="s" THEN
12250         DELETE out_file$
12260         EXIT output_loop
12270         ELSE
12280           abort_out
12290       END IF
12300     END IF
12310 :
12320   END REPeat output_loop
12330 :
12340   REMark '' Check for extension. If no ext. add _bas
12350   REPeat start_loop
12360     REMark '' IF start has been set by ENV VAR don't get input
12370     IF t_start$ <> "" THEN
12380        line_num_d = t_start$
12390        EXIT start_loop
12400     END IF
12410     REMark '' Enter Starting Line Number: (Default
12420     INK #3,4
12430     PRINT #3,lngarray$(11);d_start;")"
12440     INK #3,7
12450     INPUT #3,line_num$
12460 :
12470     INK #3,2
12480     IF line_num$ = "" THEN
12490       line_num_d = d_start
12500       EXIT start_loop
12510     ELSE
12520       REMark '' Check to make sure a number is entered.
12530       IF isnum(line_num$) THEN
12540         line_num_d = line_num$
12550         EXIT start_loop
12560       END IF
12570       BEEP 1000,10
12580       REMark '' ERROR - Must Enter A Number
12590       PRINT #3,lngarray$(12)
12600     END IF
12610     INK #3,4
12620   END REPeat start_loop
12630 :
12640   REPeat delta_loop
12650     REMark '' If delta has been set by ENV VAR then don't get input
12660     IF t_delta$ <> "" THEN
12670        line_delta = t_delta$
12680        EXIT delta_loop
12690     END IF
12700     REMark '' Enter Increment for Line Numbers: (Default
12710     INK #3,4
12720     PRINT #3,lngarray$(36);d_delta;")"
12730     INK #3,7
12740     INPUT #3,line_delta$
12750     INK #3,2
12760 :
12770     REMark '' Default for line number increment is 10.
12780     REMark '' Check to make sure a number is entered.
12790     IF line_delta$ = "" THEN
12800       line_delta = d_delta
12810       EXIT delta_loop
12820     ELSE
12830       REMark '' Check to make sure a number is entered.
12840       IF isnum(line_delta$) THEN
12850         line_delta = line_delta$
12860         EXIT delta_loop
12870       END IF
12880       BEEP 1000,10
12890       REMark '' ERROR - Must Enter A Number
12900       PRINT #3,lngarray$(12)
12910     END IF
12920     INK #3,4
12930   END REPeat delta_loop
12940 :
12950 ELSE
12960   REMark '' Command Line Arguement processing
12970   REMark '' Trim any Leading Blanks
12980 :
12990   REPeat loop_lb
13000     IF command$(1) = " " THEN
13010       command$=command$(2 TO )
13020     ELSE
13030       EXIT loop_lb
13040     END IF
13050   END REPeat loop_lb
13060 :
13070   REMark '' Trim any Trailing Blanks
13080   REPeat loop_tb
13090     x = LEN(command$)
13100     IF command$(x)=" " THEN
13110       command$=command$(1 TO x-1)
13120     ELSE
13130       EXIT loop_tb
13140     END IF
13150   END REPeat loop_tb
13160 :
13170   REMark '' Break out arguments
13180   REMark '' Check to see if there are 1 or two arguments
13190   x = " " INSTR command$
13200   IF x THEN
13210     argfile$ = command$(1 TO x-1)
13220     command$ = command$( x+1 TO )
13230     REMark '' Check for any Leading Blanks
13240     REPeat loop_lb
13250       IF command$(1) = " " THEN
13260         command$ = command$(2 TO )
13270       ELSE
13280         EXIT loop_lb
13290       END IF
13300     END REPeat loop_lb
13310     REMark '' Check to make sure line number argument is a number
13320     IF isnum(command$) = 0 THEN
13330       INK #3,2
13340       BEEP 1000,10
13350       REMark '' ERROR - 2nd Arg is Not a Valid Number
13360       PRINT #3,"      ";lngarray$(13)
13370       abort_out
13380     END IF
13390     d_start = command$
13400   ELSE
13410     argfile$ = command$
13420   END IF
13430 :
13440   in_file$ = d_working$&argfile$&d_in$
13450   out_file$ = d_work2$&argfile$&d_out$
13460 :
13470   temp = FTEST(out_file$)
13480   IF temp = 0 THEN
13490     IF d_over$="Y" OR d_over$="y" OR d_over$="S" OR d_over$="s" THEN
13500       DELETE out_file$
13510     ELSE
13520       BEEP 1000,10
13530       INK #3,2
13540       REMark '' ERROR - Output File Exists
13550       PRINT #3,"       ";lngarray$(14)
13560       abort_out
13570     END IF
13580   END IF
13590   IF temp = -11 THEN
13600     BEEP 1000,10
13610     INK #3,2
13620     REMark '' ERROR - Bad File Name - Output File
13630     PRINT #3,"       ";lngarray$(15)
13640     abort_out
13650   END IF
13660   IF temp = -9 THEN
13670     BEEP 1000,10
13680     INK #3,2
13690     REMark '' ERROR - Output File In Use
13700     PRINT #3,"       ";lngarray$(16)
13710     abort_out
13720   END IF
13730 :
13740   line_delta = d_delta
13750   line_num_d = d_start
13760 :
13770 END IF
13780 :
13790 :
13800 REMark '' Start of Main Routine
13810 line_num = line_num_d
13820 REMark '' PASS 1
13830 INK #3,4
13840 PRINT #3,"    ";lngarray$(17)
13850 :
13860 pass_one in_file$, file_num
13870 :
13880 line_num = line_num_d
13890 PRINT #3
13900 REMark '' PASS 2
13910 PRINT #3,"    ";lngarray$(18)
13920 :
13930 num_count = 0
13940 file_num = 5
13950 :
13960 REMark '' No error checking is necessary here as we have
13970 REMark '' done it earlier.
13980 OPEN_NEW #4,out_file$
13990 :
14000 pass_two in_file$, file_num
14010 :
14020 CLOSE #4
14030 PRINT #3
14040 REMark '' Program Done
14050 PRINT #3,"           ";lngarray$(19)
14060 PRINT #3,""
14070 IF OPTION_CMD$="" THEN
14080   INK #3,7
14090    REMark '' Hit Any Key
14100   PRINT #3,"           ";lngarray$(20)
14110   temp$ = INKEY$(#3,-1)
14120 END IF
14130 CLOSE #3
14140 STOP
14150 :
14160 :
14170 REMark '' The code for both pass1 and pass1 procedures have been
14180 REMark '' broken out into two different files to make it easier
14190 REMark '' to read and to maintain the code.
14200 :
14210 MISTake #INCLUDE ram1_ssbpass1_ssb
14220 :
14230 MISTake #INCLUDE ram1_ssbpass2_ssb
14240 :
14250 :
14260 :
14270 REMark '' Function first_char
14280 REMark ''   returns the location of the first non-white
14290 REMark ''   space character.
14300 DEFine FuNction first_char (a$)
14310   LOCal COUNT
14320   COUNT=0
14330 MISTake @label2
14340   COUNT=COUNT+1
14350   IF COUNT > LEN(a$) THEN RETurn 0
14360 MISTake   IF a$(count)=" " THEN GO TO @label2
14370   RETurn COUNT
14380 END DEFine first_char
14390 :
14400 :
14410 :
14420 REMark '' Function upper$
14430 REMark ''   Takes in input string and returns the
14440 REMark ''   same string in all upper case letters.
14450 DEFine FuNction UPPER$(up$)
14460   LOCal x, temp
14470 :
14480   FOR x = 1 TO LEN(up$)
14490      temp = CODE(up$(x))
14500      IF temp > 96 AND temp < 123 THEN up$(x)=CHR$(temp-32)
14510   NEXT x
14520   RETurn up$
14530 END DEFine
14540 :
14550 :
14560 :
14570 REMark '' Function isnum
14580 REMark '' Takes a string as input and determines if
14590 REMark '' if it is a number.
14600 REMark ''   Return Values:
14610 REMark ''      0 = False (is not a number)
14620 REMark ''      1 = True (is a number)
14630 DEFine FuNction isnum (var$)
14640   LOCal x,y
14650 :
14660   FOR x = 1 TO LEN(var$)
14670      y = CODE(var$(x))
14680      IF y < 48 OR y > 57 THEN RETurn 0
14690   NEXT x
14700   RETurn 1
14710 END DEFine isnum
14720 :
14730 :
14740 :
14750 REMark '' Procedure abort_out
14760 REMark ''   A separate procedure to neatly close all
14770 REMark ''   of the files and exit the program.
14780 REMark ''   Usually reserved for fatal errors.
14790 DEFine PROCedure abort_out
14800     REMark '' Close output file before any possibility of deleting it
14810 :
14820     CLOSE #4
14830 :
14840     REMark '' If called with a Command Line string, then need
14850     REMark '' to delete the output file.  This is necessary
14860     REMark '' for SSBGO so it knows when SSB has failed with
14870     REMark '' an error
14880 :
14890     IF OPTION_CMD$<>"" AND out_file$<>"" THEN DELETE out_file$
14900 :
14910     REMark '' Close out all open files
14920 :
14930     IF file_num = 5 THEN
14940         CLOSE #5
14950     ELSE
14960         FOR x = 5 TO file_num
14970             CLOSE #x
14980         NEXT x
14990     END IF
15000 :
15010     REMark '' If interactive, then pause for user input.
15020     IF OPTION_CMD$<>"" THEN
15030         PRINT #3,""
15040         INK #3,7
15050         REMark '' Hit Any Key
15060         PRINT #3,"         ";lngarray$(20)
15070         temp$ = INKEY$(#3,-1)
15080     END IF
15090         CLOSE #3
15100         STOP
15110 :
15120 END DEFine abort_out
32600 :
32601 :
32602 :
32603 :
32604 REMark ''QLC Shell Commands
32605 :
32606 DEFine PROCedure QLC_GlobalFilenameInit
32607   QLC_GlobalFilename$ = 'dos1_QLC_ssb272b_bas'
32608 END DEFine
32609 :
32610 DEFine PROCedure r
32611   RUN
32612 END DEFine
32620 DEFine PROCedure e
32621   WTV
32622   PAPER #2,0 : INK #2,4
32623 END DEFine
32630 DEFine PROCedure x
32631   s : QUIT
32632 END DEFine
32640 DEFine PROCedure q
32641   s : QUIT
32642 END DEFine
32650 DEFine PROCedure s
32651   SAVE QLC_GlobalFilename$
32652   PRINT #2, QLC_GlobalFilename$ & " SAVED"
32653 END DEFine
32660 DEFine PROCedure l
32661   LOAD QLC_GlobalFilename$
32662   PRINT #2, QLC_GlobalFilename$ & " LOADED"
32663 END DEFine
32670 DEFine PROCedure w
32671   WMOV
32672 END DEFine
32680 DEFine PROCedure u
32681   REMark DLINE 32600 TO
32682   TYPE_IN "MERGE dos1_qlc_cmd_bas" & CHR$(10)
32683 END DEFine
32690 DEFine PROCedure h
32691   PRINT #0, "CMD: h/W/r/e/x/q/s/l/u"
32692 END DEFine
